<?php

/**
 * Gravity Forms MailCamp API Library.
 *
 * @since     0.1
 * @package   GravityForms
 * @author    Imagro
 * @copyright Copyright (c) 2022, Imagro
 */
class GF_MailCamp_API {
	public $config = array();
	
	
	function __construct(){
	global $mailcampConfig;
		
		$this->config = [
			'api'		=> array(
				'url'		=> '',
			),
		];

		if(isset($mailcampConfig['url'])) 	$this->config['api']['url'] 	= $mailcampConfig['url'];
		$this->config['username'] 	= $mailcampConfig['username'];
		$this->config['token'] 		= $mailcampConfig['token'];			
	}
	
	/**
	 * Get mailing list by ID of Mailcamp
	 * @since     0.1
	 *
	 * @return array
	 */
	public function get_list( $list_id ) {
		$lists = $this->get_lists();
		
		foreach($lists as $list):
			if($list['list_id'] == $list_id) return $list;
		endforeach;
		
		return false;
	}
	
	
	/**
	 * Get mailing lists of Mailcamp
	 * @since     0.1
	 *
	 * @return array
	 */
	public function get_lists( $params = [] ) {
		
		$xml = '<xmlrequest>
					<username>'.$this->config['username'].'</username>
					<usertoken>'.$this->config['token'].'</usertoken>
					<requesttype>user</requesttype>
					<requestmethod>GetLists</requestmethod>
					<details>
						<lists />
						<countonly>false</countonly>
						<confirmedonly>true</confirmedonly>
						<sortinfo>
							<SortBy>createdate</SortBy>
							<Direction>asc</Direction>
						</sortinfo>
						<start>0</start>
						<perpage>all</perpage>
					</details>
				</xmlrequest>';
		
		
		$result = $this->curl($xml);
		$lists 	= array();
		
		if($result):
			$xml_doc = simplexml_load_string($result);

			if(isset($xml_doc->data[0])):
				foreach($xml_doc->data[0] as $item):

					$lists[] = array(
						'list_id' 	=> (int) $item->listid,
						'name' 		=> (string) $item->name,
					);

				endforeach;
			endif;
		endif;
		
		return $lists;
	}
	
	/**
	 * Get contactfields of MailCamp Mailinglist
	 * @since     0.2
	 *
	 * @return array
	 */
	public function get_list_merge_fields( $params = [] ) {
		
		$xml = '<xmlrequest>
					<username>'.$this->config['username'].'</username>
					<usertoken>'.$this->config['token'].'</usertoken>
					<requesttype>lists</requesttype>
					<requestmethod>GetCustomFields</requestmethod>
					<details>
						<listids>
							'.$params['list_id'].'
						</listids>
					</details>
				</xmlrequest>';
		
		
		$result = $this->curl($xml);
		$xml_doc = simplexml_load_string($result);

		$fields = array();			
//		$fields[] = array(
//			'field_id' => 0,
//			'name' =>  'E-mailadres',
//			'fieldtype' =>  'email',
//		);
		
		$excludeTypes = ['dropdown'];
		
		if(isset($xml_doc->data[0])):
			foreach($xml_doc->data[0] as $item):
				if(in_array($item->fieldtype, $excludeTypes)) continue;

				$fieldsettings = unserialize($item->fieldsettings);
	//			$item['fieldsettings'] = json_decode(unserialize($item['fieldsettings']), true);
	//			pr([$item, $fieldsettings]);

				if($item->fieldtype == 'checkbox' && isset($fieldsettings['Key']) && count($fieldsettings['Key']) > 0):
					foreach($fieldsettings['Key'] as $arrayKey => $key):
						$fields[] = array(
							'field_id' 		=> (string) $item->fieldid."-".$key,
							'name' 			=> (string) $item->name.": ".$fieldsettings['Value'][$arrayKey],
							'fieldtype' 	=> (string) $item->fieldtype,
							'required' 		=> ((int) $item->required == 1) ? true : false,
						);
					endforeach;
				else:
					$fields[] = array(
						'field_id' 		=> (string) $item->fieldid,
						'name' 			=> (string) $item->name,
						'fieldtype' 	=> (string) $item->fieldtype,
						'required' 		=> ((int) $item->required == 1) ? true : false,
					);
				endif;
			endforeach;
		endif;
		
		return $fields;
	}
	
	/**
	 * Check if contact exists at MailCamp Mailinglist
	 * @since     0.2
	 *
	 * @param array $list_id 	The list_id to be processed.
	 * @param array $email	 	The email address of the subscriber.
	 *
	 * @return array
	 */
	public function checkSubscriber_exists_in_Mailinglist($list_id, $email){
		if(empty($list_id) || empty($email)) return false;
		
		
		$xml = '<xmlrequest>
					<username>'.$this->config['username'].'</username>
					<usertoken>'.$this->config['token'].'</usertoken>
					<requesttype>subscribers</requesttype>
					<requestmethod>IsSubscriberOnList</requestmethod>
					<details>
						<emailaddress>'.$email.'</emailaddress>
						<listids>'.$list_id.'</listids>
					</details>
				</xmlrequest>';
		
		$result = $this->curl($xml);
		
		if($result):
			$xml_doc = simplexml_load_string($result);

			if($xml_doc->status[0] == 'SUCCESS'):
				return true;
			endif;
		endif;
		
		return false;
	}
	
	/**
	 * Gest subscriber info of MailCamp Mailinglist by Email address
	 * @since     0.2
	 *
	 * @param array $list_id 	The list_id to be processed.
	 * @param array $email	 	The email address of the subscriber.
	 *
	 * @return array
	 */
	public function getSubscriber_of_Mailinglist($list_id, $email){
		if(empty($list_id) || empty($email)) return false;
		
		
		$xml = '<xmlrequest>
					<username>'.$this->config['username'].'</username>
					<usertoken>'.$this->config['token'].'</usertoken>
					<requesttype>subscribers</requesttype>
					<requestmethod>LoadSubscriberListAll</requestmethod>
					<details>
						<listid>'.$list_id.'</listid>
					</details>
				</xmlrequest>';
		
		$result = $this->curl($xml);
		$xml_doc = simplexml_load_string($result);
		if($xml_doc->status[0] == 'SUCCESS'):
			$user = false;
		
			foreach($xml_doc->data as $item):
				$contact = $item->item;
				if($contact->emailaddress == $email):	
					$user = $item->item;
				endif;
			endforeach;
		
			return $user;
		endif;
		return false;
	}
	
	/**
	 * Add contact to MailCamp Mailinglist
	 * @since     0.2
	 *
	 * @param array $list_id 	The list_id to be processed.
	 * @param array $fields 	The fields to set to the subscriber.
	 *
	 * @return array
	 */
	public function saveSubscriber(int $list_id, array $fields = array(), array $settings = array(), $requestmethod = 'AddSubscriberToList'){
		if(empty($list_id) || count($fields) == 0) return false;
		
		
		$customFields 	= [];
		$emailField		= false;
	   	foreach ($fields as $key => $val):
			if($key == "email"):
				$emailField = $val;
			elseif(!empty($val)):
				if(strpos($key, "-")===false):
					$customFields[$key] = ['field'=>$key, 'values'=>[$val]];
				else:
					$explodeKey = explode("-", $key, 2);
					if(!isset($customFields[$explodeKey[0]])):
						$customFields[$explodeKey[0]] = ['field'=>$explodeKey[0], 'values'=>[$val]];
					else:
						$customFields[$explodeKey[0]]['values'][] = $val;
					endif;
				endif;
			endif;
	   	endforeach;
		if($emailField === false) return false;

		// Param/Settings
		$confirmed 		= (isset($settings['contact__confirmStatus'])) ? $settings['contact__confirmStatus'] : 'yes';
		$autoresponders = (isset($settings['contact__autoresponders'])) ? $settings['contact__autoresponders'] : 'false';
		
		// https://www.mailcamp.nl/api/nl/#api-Contactpersonen-AddSubscriber		
		$xml = '<xmlrequest>
					<username>'.$this->config['username'].'</username>
					<usertoken>'.$this->config['token'].'</usertoken>
					<requesttype>subscribers</requesttype>
					<requestmethod>'.$requestmethod.'</requestmethod>
					<details>
						<mailinglist>'.$list_id.'</mailinglist>
						<contact>
							<emailaddress>'.$emailField.'</emailaddress>
							<format>html</format>
							<confirmed>'.$confirmed.'</confirmed>
							<ipaddress>'.$_SERVER['REMOTE_ADDR'].'</ipaddress>
							<subscribedate>'.strtotime(date("Y-m-d H:i:s")).'</subscribedate>
							<add_to_autoresponders>'.$autoresponders.'</add_to_autoresponders>';
		
				if(is_array($customFields) && count($customFields) > 0):
					$xml .= '<customfields>';
					foreach($customFields as $field):
						$xml .= '<item>';
							$xml .= '<fieldid>'.$field['field'].'</fieldid>';
							foreach($field['values'] as $value):
								$xml .= '<value><![CDATA['.trim($value).']]></value>';
							endforeach;
						$xml .= '</item>';
					endforeach;		
					$xml .= '</customfields>';
				endif;
		
		$xml .= '
							
						</contact>
					</details>
				</xmlrequest>';
		
		$result = $this->curl($xml);
		$xml_doc = simplexml_load_string($result);
		
		if((string) $xml_doc->status == 'SUCCESS'):
			return true;
		endif;
		return false;
	}
		
	/**
	 * Modify contact to MailCamp Mailinglist
	 * @since     0.2
	 *
	 * @param array $email 		The email address of the subscriber.
	 * @param array $list_id 	The list_id to be processed.
	 * @param array $fields 	The fields to set to the subscriber.
	 *
	 * @return array
	 */
	public function updateSubscriber(string $email, int $list_id, array $fields = [], array $settings = []){
		
		// Get current values
		$subscriber = $this->getSubscriber_of_Mailinglist($list_id, $email);
		
		$updateFields = [];
		foreach($subscriber->item->item as $f):
			if($f->fieldtype == 'checkbox'):
				$currentData = unserialize($f->data);
				foreach($currentData as $cdCheckbox):
					$key = (int) $f->fieldid;
					$key .= '-'.$cdCheckbox;
					$fields[$key] = $cdCheckbox;
				endforeach;	
			elseif(isset($f->data)):
				$updateFields[(int) $f->fieldid] = (string) $f->data;
			endif;
		endforeach;
		
		foreach($fields as $k => $v):
			$updateFields[$k] = $v;
		endforeach;		
		
		return $this->saveSubscriber($list_id, $updateFields, $settings, 'EditSubscriberCustomFields');
	}
		
	/**
	 * Add contact to MailCamp Mailinglist
	 * @since     0.2
	 *
	 * @param array $list_id 	The list_id to be processed.
	 * @param array $fields 	The fields to set to the subscriber.
	 *
	 * @return array
	 */
	public function addSubscriber(int $list_id, array $fields = [], array $settings = []){
		return $this->saveSubscriber($list_id, $fields, $settings, 'AddSubscriberToList');
	}

	
	private function curl($xml){
		if(empty($xml)) return false;
				
		$ch = curl_init($this->config['api']['url']);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
		
        $response = curl_exec($ch);
		$httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		
        curl_close($ch);
		
		if(in_array($httpcode, [301, 302, 403, 404, 500])):
			return false;
		endif;
		
		return $response;
		
	}
}
