<?php

/**
 * Plugin Name: Imagro - Gravity Forms Mailcamp add-on
 * Plugin URI: https://www.imagro.nl/online
 * Description: Integrates Gravity Forms with Mailcamp, making it possible to automatically handle submits workflows.
 * Version: 1.0.6
 * Author: Imagro
 * Authors: Jeroen Caspers
 * Author URI: https://www.imagro.nl/online
 * Domain Path: /lang
 * Slug: mailcamp-imagro
 * Requires at least: 5.6
 * Tested up to: 6.3
 * Requires PHP: 7.2
 */

// plugin directory
if(!defined("IMAGRO_MAILCAMP_FILE"))	define("IMAGRO_MAILCAMP_FILE", __FILE__);
if(!defined("AUTO_UPDATE_URL")) 		define("AUTO_UPDATE_URL", 'https://wordpress.yelloobox.com/applications/wordpress/plugin-updates/imagro-mailcamp/info.json.php');

$dir = plugin_dir_path(__FILE__);

include_once($dir.'/hooks.php');
include_once($dir.'/config.php');

// load all custom functions
foreach (glob(plugin_dir_path(__FILE__) . "includes/*.php") as $file) {
    include_once $file;
}

function load_customMailcamp_wp_admin_style(){
	$dir = plugin_dir_url(__FILE__);
	wp_enqueue_style('custom-gf-mailcamp', $dir . 'assets/css/gf_settings.css', "", '0.2', 'all');
	wp_enqueue_script('custom-gf-mailcamp', $dir . 'assets/js/mailcamp.js', ['jquery'], '0.2', 'all');
}
add_action('admin_enqueue_scripts', 'load_customMailcamp_wp_admin_style');


// If Gravity Forms is loaded, bootstrap the Mailcamp Add-On.
add_action( 'gform_loaded', array( 'GF_MailCamp_Bootstrap', 'load' ), 5 );

/**
 * Class GF_MailChimp_Bootstrap
 *
 * Handles the loading of the Mailchimp Add-On and registers with the Add-On Framework.
 */
class GF_MailCamp_Bootstrap {

	function __construct(){
		
		// Auto updater
		if($this->env('localhost')){
			require (__DIR__ . '/plugin-updater/update-engine.php');
			add_filter( 'site_transient_update_plugins', 'imagro_mailcamp_push_update' );
		}
		
	}
	
	/**
	 * If the Feed Add-On Framework exists, Mailcamp Add-On is loaded.
	 *
	 * @access public
	 * @static
	 */
	public static function load() {

		if ( ! method_exists( 'GFForms', 'include_feed_addon_framework' ) ) {
			return;
		}

		require_once( 'gravityforms/class-gf-mailcamp.php' );
		

		GFAddOn::register( 'GFMailCamp' );
	}

    /**
     * Returns true if a user is logged in & is a member of the imagro team.
	 *
	 * @access private
     */
    private function env($env = false){
		$environment = 'live';
		if(stristr($_SERVER['HTTP_HOST'], '.localhost')) 	$environment = 'localhost';
		if(stristr($_SERVER['HTTP_HOST'], '.acc.')) 		$environment = 'acc';
		
		if($env) return ($environment == $env);
		return $environment;
		
    }
}

/**
 * Returns an instance of the GFMailChimp class
 *
 * @see    GFMailChimp::get_instance()
 *
 * @return GFMailChimp
 */
function gf_mailcamp() {
	return GFMailCamp::get_instance();
}